from django.conf.urls import url

from .views import ajaximage

urlpatterns = [
    url(
        r'^upload/(?P<upload_to>.*)/(?P<max_width>\d+)/(?P<max_height>\d+)/(?P<crop>\d+)/(?P<storage>[\w\-.]+)?/',
        ajaximage,
        name='ajaximage',
    )
]



import json

from django.contrib.contenttypes.admin import GenericTabularInline
from django.contrib.admin import ModelAdmin
from django.urls import reverse


class ImageInline(GenericTabularInline):
    model = None
    extra = 0


# noinspection PyProtectedMember
class AjaxImageUploadMixin(ModelAdmin):
    change_form_template = 'ajaximage/change_form.html'
    ajax_change_form_template_extends = 'admin/change_form.html'
    image_inline = ImageInline
    upload_to = '/images/'

    def _get_context(self, request, object_id, extra_context):
        if extra_context is None:
            extra_context = {}
        data = []
        obj = self.get_object(request, object_id)
        formsets, inlines = self._create_formsets(request, obj, obj is not None)
        for inline, formset in zip(inlines, formsets):
            if hasattr(inline, 'ajax_image_upload_field'):
                upload_to = self._get_inline_upload_to(inline)
                max_width, max_height, crop, storage = self._get_inline_parameters(inline)
                url = reverse(
                    'ajaximage',
                    kwargs={
                        'upload_to': upload_to,
                        'max_width': max_width,
                        'max_height': max_height,
                        'crop': crop,
                        'storage': storage,
                    },
                )
                data.append({'ajaximage_url': url, 'prefix': formset.prefix})
        extra_context.update(
            {
                'ajax_change_form_template_extends': self.ajax_change_form_template_extends,
                'data': json.dumps(data, ensure_ascii=False),
            }
        )
        return extra_context

    @staticmethod
    def _get_inline_parameters(inline):
        max_width = getattr(inline, 'ajax_image_max_width', 0)
        assert isinstance(max_width, int), 'ajax_image_max_width must be an integer!'
        max_height = getattr(inline, 'ajax_image_max_height', 0)
        assert isinstance(max_height, int), 'ajax_image_max_height must be an integer!'
        crop = getattr(inline, 'ajax_image_crop', 0)
        assert isinstance(crop, int), 'ajax_image_crop must be an integer!'
        storage = getattr(inline, 'ajax_image_storage_path', '')
        return max_width, max_height, crop, storage

    @staticmethod
    def _get_inline_upload_to(inline):
        field = inline.model._meta.get_field(getattr(inline, 'ajax_image_upload_field'))
        upload_to = None
        if field:
            if callable(field.upload_to):
                upload_to = field.upload_to(inline.model, '').strip('/')
            else:
                upload_to = field.upload_to
        return upload_to

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = self._get_context(request, object_id, extra_context=extra_context)
        return super().change_view(request, object_id, form_url='', extra_context=extra_context)

    def add_view(self, request, form_url='', extra_context=None):
        extra_context = self._get_context(request, None, extra_context=extra_context)
        return super().add_view(request, form_url='', extra_context=extra_context)

    class Media:
        js = ('admin/js/jquery.init.js', 'dropzone/js/dropzone.js')
        css = {'all': ('dropzone/css/dropzone.css', 'dropzone/css/basic.css')}



import os

from django.forms import widgets
from django.utils.safestring import mark_safe
from django.urls import reverse
from django.core.files.storage import get_storage_class


class AjaxImageWidget(widgets.TextInput):

    html = """
    <div class="ajaximage">
        <a class="file-link" target="_blank" href="{file_url}">
            <img class="file-img" src="{file_url}">
        </a>
        <a class="file-remove" href="#remove">Remove</a>
        <input class="file-path" type="hidden" value="{file_path}" id="{element_id}" name="{name}"/>
        <input type="file" class="file-input" />
        <input class="file-dest" type="hidden" value="{upload_url}">
        <div class="progress progress-striped active">
            <div class="bar"></div>
        </div>
    </div>
    """

    class Media:
        js = ('ajaximage/js/ajaximage.js',)
        css = {'all': ('ajaximage/css/bootstrap-progress.min.css', 'ajaximage/css/styles.css')}

    def __init__(self, *args, **kwargs):
        self.upload_to = kwargs.pop('upload_to', '')
        self.max_width = kwargs.pop('max_width', 0)
        self.max_height = kwargs.pop('max_height', 0)
        self.crop = kwargs.pop('crop', 0)
        self.storage_path = kwargs.pop('storage_path', None)

        super(AjaxImageWidget, self).__init__(*args, **kwargs)

    # noinspection PyMethodOverriding
    def render(self, name, value, attrs=None, renderer=None):
        final_attrs = self.build_attrs(attrs)
        element_id = final_attrs.get('id')
        upload_to = self.upload_to
        if callable(self.upload_to):
            upload_to = self.upload_to(None, '')
        kwargs = {
            'upload_to': upload_to,
            'max_width': self.max_width,
            'max_height': self.max_height,
            'crop': self.crop,
            'storage': self.storage_path if self.storage_path is not None else '',
        }
        upload_url = reverse('ajaximage', kwargs=kwargs)
        file_path = str(value) if value else ''
        storage = get_storage_class(self.storage_path)()
        file_url = storage.url(file_path) if value else ''
        file_name = os.path.basename(file_url)
        output = self.html.format(
            upload_url=upload_url,
            file_url=file_url,
            file_name=file_name,
            file_path=file_path,
            element_id=element_id,
            name=name,
        )

        return mark_safe(output)
